@@ -6,3 +6,5 @@ CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
 
 PATH_METAVAR = "<path>"
 INT_METAVAR = "<n>"
+VERSION_METAVAR = "<version>"
+MODEL_METAVAR = "<model>"
