@@ -3,8 +3,10 @@
 The functions in this module are entry points for ratom sub-commands, e.g. `ratom entities ...`
 """
 
+import json
 import logging
 from datetime import datetime
+from operator import itemgetter
 from pathlib import Path
 from typing import Optional
 
@@ -13,7 +15,12 @@ from packaging.version import parse
 from sqlalchemy import func
 
 from libratom.cli.utils import MockContext, install_spacy_model, list_spacy_models
-from libratom.lib.core import get_set_of_files, get_spacy_models, load_spacy_model
+from libratom.lib.core import (
+    export_messages_from_file,
+    get_set_of_files,
+    get_spacy_models,
+    load_spacy_model,
+)
 from libratom.lib.database import db_init, db_session
 from libratom.lib.entities import extract_entities
 from libratom.lib.report import generate_report, scan_files, store_configuration_in_db
@@ -25,7 +32,12 @@ OUTPUT_FILENAME_TEMPLATE = "{}_{}_{}.sqlite3"
 
 
 def entities(
-    out: Path, spacy_model_name: str, jobs: Optional[int], src: Path, progress: bool
+    out: Path,
+    spacy_model_name: str,
+    jobs: Optional[int],
+    src: Path,
+    include_message_contents: bool = False,
+    progress: bool = False,
 ) -> int:
     """
     Click sub command function called by `ratom entities`
@@ -122,6 +134,7 @@ def entities(
                 files=good_files,
                 session=session,
                 spacy_model=spacy_model,
+                include_message_contents=include_message_contents,
                 jobs=jobs,
                 processing_progress_callback=processing_msg_bar.update,
                 reporting_progress_callback=reporting_msg_bar.update,
@@ -132,7 +145,13 @@ def entities(
     return status
 
 
-def report(out: Path, jobs: Optional[int], src: Path, progress: bool) -> int:
+def report(
+    out: Path,
+    jobs: Optional[int],
+    src: Path,
+    include_message_contents: bool = False,
+    progress: bool = False,
+) -> int:
     """
     Click sub command function called by `ratom report`
     """
@@ -202,7 +221,10 @@ def report(out: Path, jobs: Optional[int], src: Path, progress: bool) -> int:
         ) as msg_bar:
 
             status = generate_report(
-                files=good_files, session=session, progress_callback=msg_bar.update,
+                files=good_files,
+                session=session,
+                include_message_contents=include_message_contents,
+                progress_callback=msg_bar.update,
             )
 
     logger.info("All done")
@@ -223,3 +245,29 @@ def model(_list: bool, install: str, upgrade: str, version: str) -> int:
 
     # List by default
     return list_spacy_models()
+
+
+def emldump(out: Path, location: Path, src: Path) -> int:
+    """
+    Generate .eml files from pst/mbox files.
+    """
+
+    # Extract inputs from json file
+    with src.open() as f:
+        input_elements = json.load(f)
+
+    # Process each file / id_list
+    for input_element in input_elements:
+
+        filename, id_list = itemgetter("filename", "id_list")(input_element)
+
+        try:
+            export_messages_from_file(location / filename, id_list, out)
+
+        except Exception as exc:
+            logger.warning(
+                f"Skipping {location / filename}, reason: {exc}",
+                exc_info=True,
+            )
+
+    return 0
