@@ -15,11 +15,8 @@ from sqlalchemy.orm.session import Session
 
 import libratom
 from libratom.lib.concurrency import get_messages, imap_job, worker_init
-from libratom.lib.core import (
-    RATOM_MSG_PROGRESS_STEP,
-    get_ratom_settings,
-    open_mail_archive,
-)
+from libratom.lib.core import get_ratom_settings, open_mail_archive
+from libratom.lib.utils import cleanup_message_body
 from libratom.models import Attachment, Configuration, FileReport, Message
 
 logger = logging.getLogger(__name__)
@@ -145,6 +142,7 @@ def store_configuration_in_db(
 def generate_report(
     files: Iterable[Path],
     session: Session,
+    include_message_contents: bool = False,
     progress_callback: Optional[Callable] = None,
 ) -> int:
     """
@@ -161,13 +159,13 @@ def generate_report(
     # Load the file_report table for local lookup
     _file_reports = session.query(FileReport).all()  # noqa: F841
 
-    msg_count = 0
-
     try:
 
-        for msg_count, msg_info in enumerate(
-            get_messages(files, progress_callback=update_progress, with_content=False),
-            start=1,
+        for msg_info in get_messages(
+            files,
+            progress_callback=update_progress,
+            with_content=include_message_contents,
+            with_headers=include_message_contents,
         ):
 
             # Extract results
@@ -175,6 +173,11 @@ def generate_report(
             filepath = msg_info.pop("filepath")
             attachments = msg_info.pop("attachments")
 
+            if include_message_contents:
+                msg_info["body"] = cleanup_message_body(
+                    msg_info["body"], msg_info.pop("body_type")
+                )
+
             # Create new message instance
             message = Message(pff_identifier=message_id, **msg_info)
 
@@ -202,13 +205,6 @@ def generate_report(
                 ]
             )
 
-            # Update progress every N messages
-            if not msg_count % RATOM_MSG_PROGRESS_STEP:
-                update_progress(RATOM_MSG_PROGRESS_STEP)
-
-        # Update progress with remaining message count
-        update_progress(msg_count % RATOM_MSG_PROGRESS_STEP)
-
     except KeyboardInterrupt:
         logger.warning("Cancelling running task")
         logger.info("Partial results written to database")
