@@ -1,25 +1,38 @@
-# pylint: disable=missing-docstring,invalid-name,too-few-public-methods,misplaced-comparison-constant,no-value-for-parameter
+# pylint: disable=missing-docstring,invalid-name,too-few-public-methods,misplaced-comparison-constant
 
 import datetime
 import os
 import tempfile
+from contextlib import contextmanager
 from pathlib import Path
 from typing import List, Optional, Union
 
 import click
 import pytest
 from click.testing import CliRunner, Result
-from sqlalchemy import create_engine
+from sqlalchemy import create_engine, func
 from sqlalchemy.orm import sessionmaker
 
 import libratom
 import libratom.cli.subcommands as subcommands
 from libratom.cli.cli import ratom
-from libratom.cli.utils import list_spacy_models, validate_version_string
-from libratom.lib.core import SPACY_MODELS, load_spacy_model
-from libratom.lib.database import db_session
+from libratom.cli.utils import (
+    list_spacy_models,
+    validate_eml_export_input,
+    validate_existing_dir,
+    validate_version_string,
+)
+from libratom.lib.constants import SPACY_MODELS
+from libratom.lib.core import load_spacy_model, open_mail_archive
+from libratom.lib.database import db_session_from_cmd_out
 from libratom.lib.entities import process_message
-from libratom.models import Entity, FileReport
+from libratom.lib.utils import BodyType, cleanup_message_body
+from libratom.models import Configuration, Entity, FileReport, Message
+
+
+@contextmanager
+def does_not_raise():
+    yield
 
 
 class Expected:
@@ -27,10 +40,13 @@ class Expected:
     Result object type for parametrized tests. Expand as necessary...
     """
 
-    def __init__(self, status: int, tokens: List[str]):
+    def __init__(self, status: int, tokens: List[str], **kwargs):
         self.status = status
         self.tokens = tokens
 
+        for key, value in kwargs.items():
+            setattr(self, key, value)
+
 
 def run_ratom_subcommand(
     cmd: str,
@@ -102,6 +118,19 @@ def manage_spacy_models(
     return run_ratom_subcommand("model", options, args, runner, expected)
 
 
+def dump_eml_files(
+    options: List,
+    args: Union[Path, str, None],
+    runner: CliRunner,
+    expected: Optional[Expected],
+) -> Result:
+    """
+    Block of code to run an eml dump job as part of a test
+    """
+
+    return run_ratom_subcommand("emldump", options, args, runner, expected)
+
+
 @pytest.mark.parametrize(
     "params, expected",
     [
@@ -117,6 +146,9 @@ def test_ratom(cli_runner, params, expected):
     assert result.exit_code == 0
     assert expected in result.output
 
+    with pytest.raises(ValueError):
+        db_session_from_cmd_out(result)
+
 
 @pytest.mark.parametrize(
     "params, expected",
@@ -142,7 +174,7 @@ def test_ratom_entities(cli_runner, params, expected):
     "params, expected",
     [
         (
-            ["-vvp", "-j2"],
+            ["-vvpm", "-j2"],
             Expected(status=0, tokens=["Creating database file", "All done"]),
         )
     ],
@@ -150,7 +182,24 @@ def test_ratom_entities(cli_runner, params, expected):
 def test_ratom_entities_enron_001(
     isolated_cli_runner, enron_dataset_part001, params, expected
 ):
-    extract_entities(params, enron_dataset_part001, isolated_cli_runner, expected)
+    msg_id = 2097572
+
+    # Run entity extraction job with message content flag on
+    result = extract_entities(
+        params, enron_dataset_part001, isolated_cli_runner, expected
+    )
+
+    # Get message contents from DB
+    with db_session_from_cmd_out(result) as session:
+        msg = session.query(Message).filter_by(pff_identifier=msg_id).one()
+        headers, body = msg.headers, msg.body
+
+    # Access message directly and compare
+    archive_file = list(enron_dataset_part001.glob("*.pst"))[0]
+    with open_mail_archive(archive_file) as archive:
+        message = archive.get_message_by_id(msg_id)
+        assert cleanup_message_body(*archive.get_message_body(message)) == body
+        assert archive.get_message_headers(message) == headers
 
 
 @pytest.mark.parametrize(
@@ -158,18 +207,55 @@ def test_ratom_entities_enron_001(
     [
         (
             ["-vvp", "-j2"],
-            Expected(status=0, tokens=["Creating database file", "All done"]),
-        )
+            Expected(
+                status=0,
+                tokens=["Creating database file", "All done"],
+                with_messages=False,
+            ),
+        ),
+        (
+            ["-vvpm", "-j2"],
+            Expected(
+                status=0,
+                tokens=["Creating database file", "All done"],
+                with_messages=True,
+            ),
+        ),
     ],
 )
-def test_ratom_report_enron_001(
-    isolated_cli_runner, enron_dataset_part001, params, expected
+def test_ratom_report_enron_027(
+    isolated_cli_runner, enron_dataset_part027, params, expected
 ):
-    generate_report(params, enron_dataset_part001, isolated_cli_runner, expected)
+    msg_id = 2390436
+
+    result = generate_report(
+        params, enron_dataset_part027, isolated_cli_runner, expected
+    )
+
+    with db_session_from_cmd_out(result) as session:
+        # Verify total message count
+        assert session.query(Message).count() == 9297
+
+        # Get message contents from DB
+        msg = session.query(Message).filter_by(pff_identifier=msg_id).one()
+        headers, body = msg.headers, msg.body
+
+        if expected.with_messages:
+            # Access message directly and compare
+            archive_file = list(enron_dataset_part027.glob("*.pst"))[0]
+            with open_mail_archive(archive_file) as archive:
+                message = archive.get_message_by_id(msg_id)
+                assert cleanup_message_body(*archive.get_message_body(message)) == body
+                assert archive.get_message_headers(message) == headers
+
+        else:
+            assert headers is None
+            assert body is None
 
 
 @pytest.mark.parametrize(
-    "params, expected", [(["-v"], Expected(status=0, tokens=["No PST file found"]))],
+    "params, expected",
+    [(["-v"], Expected(status=0, tokens=["No PST file found"]))],
 )
 def test_ratom_report_empty(isolated_cli_runner, params, expected):
     # Make new empty dir
@@ -231,8 +317,9 @@ def test_ratom_entities_from_mbox_files(
 
 
 @pytest.mark.skipif(
-    not os.getenv("CONTINUOUS_INTEGRATION", None),
-    reason="Keep local test runs reasonably short",
+    not os.getenv("CONTINUOUS_INTEGRATION", None)
+    or os.getenv("TRAVIS_OS_NAME", None) == "osx",
+    reason="Keep local test runs and OSX travis runs reasonably short",
 )
 @pytest.mark.parametrize(
     "params, expected",
@@ -244,60 +331,63 @@ def test_ratom_entities_from_mbox_files(
     ],
 )
 def test_ratom_entities_enron_004(
-    isolated_cli_runner, enron_dataset_part004, params, expected
+    isolated_cli_runner,
+    enron_dataset_part004,
+    en_core_web_sm_2_3_1,  # pylint: disable=unused-argument
+    params,
+    expected,
 ):
     result = extract_entities(
         params, enron_dataset_part004, isolated_cli_runner, expected
     )
 
-    # Validate output file
-    db_file = None
-    for line in result.output.splitlines():
-        if line.startswith("Creating database file:"):
-            db_file = Path(line.rsplit(maxsplit=1)[1].strip())
-
-    assert db_file.is_file()
-
-    # Open DB session
-    engine = create_engine(f"sqlite:///{db_file}")
-    Session = sessionmaker(bind=engine)
-
-    with db_session(Session) as session:
+    with db_session_from_cmd_out(result) as session:
 
         # Sanity check
         for entity in session.query(Entity)[:10]:
             assert str(entity)
 
         # Verify total entity count
-        assert session.query(Entity).count() >= 100_000
+        assert session.query(Entity).count() == 173_736
 
-        # Verify entity types found
-        results = session.query(Entity.label_).all()
+        # Verify count per entity type
+        results = (
+            session.query(Entity.label_, func.count(Entity.label_))
+            .group_by(Entity.label_)
+            .all()
+        )
 
-        # Expected entity types
-        entity_types = {
-            "CARDINAL",
-            "DATE",
-            "EVENT",
-            "FAC",
-            "GPE",
-            "LANGUAGE",
-            "LAW",
-            "LOC",
-            "MONEY",
-            "NORP",
-            "ORDINAL",
-            "ORG",
-            "PERCENT",
-            "PERSON",
-            "PRODUCT",
-            "QUANTITY",
-            "TIME",
-            "WORK_OF_ART",
+        assert results
+
+        expected_counts = {
+            "CARDINAL": 25984,
+            "DATE": 9947,
+            "EVENT": 1490,
+            "FAC": 1441,
+            "GPE": 10749,
+            "LANGUAGE": 4,
+            "LAW": 12953,
+            "LOC": 371,
+            "MONEY": 1900,
+            "NORP": 559,
+            "ORDINAL": 593,
+            "ORG": 84406,
+            "PERCENT": 1070,
+            "PERSON": 15865,
+            "PRODUCT": 1400,
+            "QUANTITY": 553,
+            "TIME": 3566,
+            "WORK_OF_ART": 885,
         }
 
-        # Confirm all types have been found
-        assert entity_types == {entity_type for entity_type, in results}
+        for entity_type, count in results:
+            assert expected_counts[entity_type] == count
+
+        # Confirm spaCy model version for this job
+        config = (
+            session.query(Configuration).filter_by(name="spacy_model_version").one()
+        )
+        assert config.value == "2.3.1"
 
 
 @pytest.mark.skipif(
@@ -365,8 +455,8 @@ def test_file_report(enron_dataset_part012):
         engine = create_engine(f"sqlite:////{out}")
         session = sessionmaker(bind=engine)()
 
-        # There should be one FileReport instance fir this run
-        file_report = session.query(FileReport).one()
+        # There should be one FileReport instance for this run
+        file_report = session.query(FileReport).one()  # pylint: disable=no-member
 
         # Path
         assert file_report.path == str(file)
@@ -396,13 +486,17 @@ def test_file_report(enron_dataset_part012):
 
 
 def test_process_message():
+    # pylint:disable=no-value-for-parameter
+
     filepath, message_id = "/foo/bar", 1234
     res, error = process_message(
         # Must use dictionary form if function is called explicitly
         {
             "filepath": filepath,
             "message_id": message_id,
-            "message": "hello",
+            "date": datetime.datetime.utcnow(),
+            "body": "hello",
+            "body_type": BodyType.PLAIN,
             "spacy_model": None,
             "attachments": None,
         }
@@ -418,12 +512,48 @@ def test_list_spacy_models():
     assert list_spacy_models() == 0
 
 
-@pytest.mark.parametrize("value", ["1.2.3", "2.0", "0.0a1", None])
-def test_validate_version_string(value):
-    assert validate_version_string(None, None, value) == value
+@pytest.mark.parametrize(
+    "value,context",
+    [
+        ("1.2.3", does_not_raise()),
+        ("2.0", does_not_raise()),
+        ("0.0a1", does_not_raise()),
+        (None, does_not_raise()),
+        ("foobar", pytest.raises(click.BadParameter)),
+        ("1", pytest.raises(click.BadParameter)),
+        (".5", pytest.raises(click.BadParameter)),
+    ],
+)
+def test_validate_version_string(value, context):
+    with context:
+        assert validate_version_string(None, None, value) == value
 
 
-@pytest.mark.parametrize("value", ["foobar", "1", ".5"])
-def test_validate_version_string_with_bad_versions(value):
+@pytest.mark.parametrize(
+    "value,context",
+    [
+        (Path.cwd(), does_not_raise()),
+        (Path("/bad/path/"), pytest.raises(click.BadParameter)),
+    ],
+)
+def test_validate_existing_dir(value, context):
+    with context:
+        assert validate_existing_dir(None, None, value) == value
+
+
+def test_validate_eml_export_input(bad_eml_export_input):
     with pytest.raises(click.BadParameter):
-        validate_version_string(None, None, value)
+        validate_eml_export_input(None, None, bad_eml_export_input)
+
+
+def test_ratom_emldump(cli_runner, enron_dataset_part004, good_eml_export_input):
+    expected = Expected(status=0, tokens=[])
+
+    with tempfile.TemporaryDirectory() as tmpdir:
+        params = [
+            f"-l{enron_dataset_part004}",
+            f"-o{tmpdir}",
+            str(good_eml_export_input),
+        ]
+
+        dump_eml_files(params, None, cli_runner, expected)
