@@ -1,22 +1,20 @@
-# pylint: disable=missing-docstring,invalid-name,no-member,unused-import
+# pylint: disable=missing-docstring,invalid-name,protected-access
 import email
 import hashlib
-import json
 import logging
-import os
 from email import policy
 from pathlib import Path
 from tempfile import TemporaryDirectory
-from unittest.mock import MagicMock, patch
+from unittest.mock import MagicMock, Mock, patch
 
 import pytest
-import requests
 
 import libratom
-from libratom import data
+from libratom.data import MIME_TYPES
 from libratom.lib.concurrency import get_messages
+from libratom.lib.constants import SPACY_MODELS, BodyType
 from libratom.lib.core import (
-    SPACY_MODELS,
+    extract_message_from_archive,
     get_set_of_files,
     load_spacy_model,
     open_mail_archive,
@@ -27,19 +25,27 @@ from libratom.lib.entities import extract_entities
 from libratom.lib.exceptions import FileTypeError
 from libratom.lib.mbox import MboxArchive
 from libratom.lib.pff import PffArchive
-from libratom.lib.report import get_file_info, scan_files
+from libratom.lib.report import generate_report, get_file_info, scan_files
+from libratom.lib.utils import cleanup_message_body
 from libratom.models import FileReport
 
-try:
-    from importlib import resources
-except ImportError:
-    # backport version for Python 3.6
-    import importlib_resources as resources
-
-
 logger = logging.getLogger(__name__)
 
 
+class BadPffFolder(MagicMock):
+    """
+    Mock folder class to simulate libpff errors
+    """
+
+    @property
+    def number_of_sub_messages(self):
+        raise OSError
+
+    @property
+    def sub_messages(self):
+        raise OSError
+
+
 def test_version():
     assert libratom.__version__
 
@@ -92,7 +98,8 @@ def test_get_transport_headers_from_sent_items(enron_dataset_part004):
                 try:
                     name = folder.name.lower()
                 except AttributeError:
-                    if folder.identifier != archive.data.root_folder.identifier:
+                    # pylint: disable=no-member
+                    if folder.identifier != archive._data.root_folder.identifier:
                         raise
                     continue
                 if "sent mail" in name or "sent items" in name:
@@ -101,8 +108,7 @@ def test_get_transport_headers_from_sent_items(enron_dataset_part004):
 
 
 def test_extract_message_attachments(enron_dataset_part002):
-    """Checking 3 known attachments, to validate the attachment extraction process
-    """
+    """Checking 3 known attachments, to validate the attachment extraction process"""
 
     digests = {
         47685: "d48232614b01e56014293854abbb5db3",
@@ -163,47 +169,6 @@ def test_get_message_by_id_with_bad_id(sample_pst_file):
         assert archive.get_message_by_id(1234) is None
 
 
-def test_get_messages_with_bad_messages(enron_dataset_part012, mock_progress_callback):
-
-    _count = 0
-    for _count, res in enumerate(
-        get_messages(
-            files=enron_dataset_part012.glob("*.pst"),
-            progress_callback=mock_progress_callback,
-        ),
-        start=1,
-    ):
-        assert res
-
-    assert _count == 11262
-
-
-@pytest.mark.skipif(
-    not os.getenv("CONTINUOUS_INTEGRATION", None)
-    or os.getenv("TRAVIS_OS_NAME", None) == "osx",
-    reason="Keep local test runs and OSX travis runs reasonably short",
-)
-def test_extract_entities_with_bad_messages(enron_dataset_part012):
-
-    tmp_filename = "test.sqlite3"
-
-    with TemporaryDirectory() as tmpdir:
-
-        destination = Path(tmpdir) / tmp_filename
-        Session = db_init(destination)
-
-        with db_session(Session) as session:
-
-            status = extract_entities(
-                files=enron_dataset_part012.glob("*.pst"),
-                session=session,
-                spacy_model=load_spacy_model(SPACY_MODELS.en_core_web_sm)[0],
-                jobs=2,
-            )
-
-        assert status == 0
-
-
 def test_file_report_with_empty_relationship():
     file_report = FileReport()
 
@@ -233,7 +198,23 @@ def test_extract_entities_from_mbox_files(directory_of_mbox_files):
         assert status == 0
 
 
-def test_extract_entities_with_interrupt(directory_of_mbox_files):
+@pytest.mark.parametrize(
+    "function, patched, kwargs",
+    [
+        (
+            extract_entities,
+            "libratom.lib.entities.Message",
+            {
+                "spacy_model": load_spacy_model(SPACY_MODELS.en_core_web_sm)[0],
+                "jobs": 2,
+            },
+        ),
+        (generate_report, "libratom.lib.report.Message", {}),
+    ],
+)
+def test_run_function_with_interrupt(
+    directory_of_mbox_files, function, patched, kwargs
+):
 
     tmp_filename = "test.sqlite3"
 
@@ -243,15 +224,14 @@ def test_extract_entities_with_interrupt(directory_of_mbox_files):
         Session = db_init(destination)
 
         with db_session(Session) as session, patch(
-            "libratom.lib.entities.Message",
+            patched,
             new=MagicMock(side_effect=KeyboardInterrupt),
         ):
 
-            status = extract_entities(
+            status = function(
                 files=get_set_of_files(directory_of_mbox_files),
                 session=session,
-                spacy_model=load_spacy_model(SPACY_MODELS.en_core_web_sm)[0],
-                jobs=2,
+                **kwargs,
             )
 
         assert status == 1
@@ -331,31 +311,79 @@ def test_get_file_info(sample_pst_file):
 
 def test_attachments_mime_type_validation(enron_dataset, mock_progress_callback):
 
-    # Load media types
-    with resources.path(data, "media_types.json") as media_types_file, open(
-        media_types_file, "r"
-    ) as f:
-        media_types = json.load(f)
-
-    # Some enron files have these non-official attachment types
-    media_types.extend(["application/msexcell", "application/mspowerpoint"])
-
     files = get_set_of_files(enron_dataset)
 
     for res in get_messages(files, progress_callback=mock_progress_callback):
         attachments = res.get("attachments")
         if attachments:
             for attachment in attachments:
-                assert attachment.mime_type in media_types
+                try:
+                    assert attachment.mime_type in MIME_TYPES
+                except AssertionError:
+                    # Some enron files have these obsolete attachment types
+                    assert attachment.mime_type in [
+                        "application/msexcell",
+                        "application/mspowerpoint",
+                    ]
 
 
 def test_get_mbox_message_by_id(sample_mbox_file):
     with open_mail_archive(sample_mbox_file) as archive:
+
+        assert archive.message_count == 113
+
         for index, message in enumerate(archive.messages(), start=1):
             msg = archive.get_message_by_id(index)
+            assert extract_message_from_archive(archive, index)
             assert archive.format_message(msg) == archive.format_message(message)
+            assert archive.get_message_headers(message)
 
 
 def test_get_mbox_message_by_id_with_bad_id(sample_mbox_file):
     with open_mail_archive(sample_mbox_file) as archive:
         assert archive.get_message_by_id(1234) is None
+
+
+@pytest.mark.parametrize("mock_cls", [MagicMock, Mock])
+def test_get_attachment_metadata(mock_cls):
+    message = MagicMock(identifier=123, attachments=[mock_cls(name="foo", size="0")])
+
+    assert PffArchive().get_attachment_metadata(message)[0].mime_type is None
+
+
+@pytest.mark.parametrize(
+    "message, body_type",
+    [
+        (Mock(), BodyType.PLAIN),
+        (Mock(plain_text_body=None), BodyType.RTF),
+        (Mock(plain_text_body=None, rtf_body=None), BodyType.HTML),
+        (Mock(plain_text_body=None, rtf_body=None, html_body=None), None),
+    ],
+)
+def test_get_message_body(message, body_type):
+    assert PffArchive().get_message_body(message)[1] is body_type
+
+
+@pytest.mark.parametrize(
+    "body, body_type, result",
+    [
+        (
+            r"{\rtf {\fonttbl {\f0 Times New Roman;}} \f0\fs60 foo} ",
+            BodyType.RTF,
+            "foo",
+        ),
+        ("<body><table><tr><td>foo</td></tr></table></body>", BodyType.HTML, "foo"),
+    ],
+)
+def test_cleanup_message_body(body, body_type, result):
+    assert cleanup_message_body(body, body_type) == result
+
+
+def test_pff_archive_with_bad_folders(sample_pst_file):
+    with PffArchive(sample_pst_file) as archive:
+        with patch.object(archive, "folders") as mock_folders:
+            mock_folders.return_value = [BadPffFolder()]
+
+            # No uncaught exception
+            assert archive.message_count == 0
+            assert not list(archive.messages())
